@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin Lite - a comprehensive, MV3-compliant content blocker
     Copyright (C) 2022-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -31,6 +31,7 @@ import {
     runtime,
     localRead, localWrite,
     sessionRead, sessionWrite,
+    adminRead,
 } from './ext.js';
 
 import {
@@ -50,10 +51,13 @@ import {
     setFilteringMode,
     getDefaultFilteringMode,
     setDefaultFilteringMode,
+    getTrustedSites,
+    setTrustedSites,
     syncWithBrowserPermissions,
 } from './mode-manager.js';
 
 import {
+    broadcastMessage,
     ubolLog,
 } from './utils.js';
 
@@ -62,7 +66,7 @@ import {
 const rulesetConfig = {
     version: '',
     enabledRulesets: [ 'default' ],
-    autoReload: 1,
+    autoReload: true,
 };
 
 const UBOL_ORIGIN = runtime.getURL('').replace(/\/$/, '');
@@ -81,7 +85,7 @@ async function loadRulesetConfig() {
     if ( data ) {
         rulesetConfig.version = data.version;
         rulesetConfig.enabledRulesets = data.enabledRulesets;
-        rulesetConfig.autoReload = data.autoReload;
+        rulesetConfig.autoReload = data.autoReload && true || false;
         wakeupRun = true;
         return;
     }
@@ -89,7 +93,7 @@ async function loadRulesetConfig() {
     if ( data ) {
         rulesetConfig.version = data.version;
         rulesetConfig.enabledRulesets = data.enabledRulesets;
-        rulesetConfig.autoReload = data.autoReload;
+        rulesetConfig.autoReload = data.autoReload && true || false;
         sessionWrite('rulesetConfig', rulesetConfig);
         return;
     }
@@ -150,15 +154,14 @@ function onMessage(request, sender, callback) {
         }).catch(reason => {
             console.log(reason);
         });
-        callback();
-        return;
+        return false;
     }
 
     default:
         break;
     }
 
-    // Does requires trusted origin.
+    // Does require trusted origin.
 
     // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/runtime/MessageSender
     //   Firefox API does not set `sender.origin`
@@ -173,6 +176,7 @@ function onMessage(request, sender, callback) {
         }).then(( ) => {
             registerInjectables();
             callback();
+            broadcastMessage({ enabledRulesets: rulesetConfig.enabledRulesets });
         });
         return true;
     }
@@ -180,20 +184,23 @@ function onMessage(request, sender, callback) {
     case 'getOptionsPageData': {
         Promise.all([
             getDefaultFilteringMode(),
+            getTrustedSites(),
             getRulesetDetails(),
             dnr.getEnabledRulesets(),
         ]).then(results => {
             const [
                 defaultFilteringMode,
+                trustedSites,
                 rulesetDetails,
                 enabledRulesets,
             ] = results;
             callback({
                 defaultFilteringMode,
+                trustedSites: Array.from(trustedSites),
                 enabledRulesets,
                 maxNumberOfEnabledRulesets: dnr.MAX_NUMBER_OF_ENABLED_STATIC_RULESETS,
                 rulesetDetails: Array.from(rulesetDetails.values()),
-                autoReload: rulesetConfig.autoReload === 1,
+                autoReload: rulesetConfig.autoReload,
                 firstRun,
             });
             firstRun = false;
@@ -202,9 +209,10 @@ function onMessage(request, sender, callback) {
     }
 
     case 'setAutoReload':
-        rulesetConfig.autoReload = request.state ? 1 : 0;
+        rulesetConfig.autoReload = request.state && true || false;
         saveRulesetConfig().then(( ) => {
             callback();
+            broadcastMessage({ autoReload: rulesetConfig.autoReload });
         });
         return true;
 
@@ -217,7 +225,7 @@ function onMessage(request, sender, callback) {
         ]).then(results => {
             callback({
                 level: results[0],
-                autoReload: rulesetConfig.autoReload === 1,
+                autoReload: rulesetConfig.autoReload,
                 hasOmnipotence: results[1],
                 hasGreatPowers: results[2],
                 rulesetDetails: results[3],
@@ -261,6 +269,21 @@ function onMessage(request, sender, callback) {
         return true;
     }
 
+    case 'setTrustedSites':
+        setTrustedSites(request.hostnames).then(( ) => {
+            registerInjectables();
+            return Promise.all([
+                getDefaultFilteringMode(),
+                getTrustedSites(),
+            ]);
+        }).then(results => {
+            callback({
+                defaultFilteringMode: results[0],
+                trustedSites: Array.from(results[1]),
+            });
+        });
+        return true;
+
     default:
         break;
     }
@@ -317,7 +340,10 @@ async function start() {
     );
 
     if ( firstRun ) {
-        runtime.openOptionsPage();
+        const disableFirstRunPage = await adminRead('disableFirstRunPage');
+        if ( disableFirstRunPage !== true ) {
+            runtime.openOptionsPage();
+        }
     }
 }
 
