@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin Lite - a comprehensive, MV3-compliant content blocker
     Copyright (C) 2022-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -417,10 +417,6 @@ function registerSpecific(context) {
 /******************************************************************************/
 
 function registerScriptlet(context, scriptletDetails) {
-    // https://bugzilla.mozilla.org/show_bug.cgi?id=1736575
-    //   `MAIN` world not yet supported in Firefox
-    if ( isGecko ) { return; }
-
     const { before, filteringModeDetails, rulesetsDetails } = context;
 
     const hasBroadHostPermission =
@@ -476,9 +472,14 @@ function registerScriptlet(context, scriptletDetails) {
                 matches,
                 excludeMatches,
                 runAt: 'document_start',
-                world: 'MAIN',
             };
 
+            // https://bugzilla.mozilla.org/show_bug.cgi?id=1736575
+            //   `MAIN` world not yet supported in Firefox
+            if ( isGecko === false ) {
+                directive.world = 'MAIN';
+            }
+
             // register
             if ( registered === undefined ) {
                 context.toAdd.push(directive);
