@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2022-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -53,19 +53,23 @@ const commandLineArgs = (( ) => {
     return args;
 })();
 
+const platform = commandLineArgs.get('platform') || 'chromium';
 const outputDir = commandLineArgs.get('output') || '.';
 const cacheDir = `${outputDir}/../mv3-data`;
 const rulesetDir = `${outputDir}/rulesets`;
 const scriptletDir = `${rulesetDir}/scripting`;
 const env = [
-    'chromium',
+    platform,
     'mv3',
-    'native_css_has',
     'ublock',
     'ubol',
     'user_stylesheet',
 ];
 
+if ( platform !== 'firefox' ) {
+    env.push('native_css_has');
+}
+
 /******************************************************************************/
 
 const jsonSetMapReplacer = (k, v) => {
@@ -245,7 +249,61 @@ const isGood = rule =>
 
 /******************************************************************************/
 
-async function processNetworkFilters(assetDetails, network) {
+// Two distinct hostnames:
+//   www.example.com
+//   example.com
+// Can be reduced to a single one:
+//   example.com
+// Since if example.com matches, then www.example.com (or any other subdomain
+// of example.com) will always match.
+
+function pruneHostnameArray(hostnames) {
+    const rootMap = new Map();
+    for ( const hostname of hostnames ) {
+        const labels = hostname.split('.');
+        let currentMap = rootMap;
+        let i = labels.length;
+        while ( i-- ) {
+            const label = labels[i];
+            let nextMap = currentMap.get(label);
+            if ( nextMap === null ) { break; }
+            if ( nextMap === undefined ) {
+                if ( i === 0 ) {
+                    currentMap.set(label, (nextMap = null));
+                } else {
+                    currentMap.set(label, (nextMap = new Map()));
+                }
+            } else if ( i === 0 ) {
+                currentMap.set(label, null);
+            }
+            currentMap = nextMap;
+        }
+    }
+    const assemble = (currentMap, currentHostname, out) => {
+        for ( const [ label, nextMap ] of currentMap ) {
+            const nextHostname = currentHostname === ''
+                ? label
+                : `${label}.${currentHostname}`;
+            if ( nextMap === null ) {
+                out.push(nextHostname);
+            } else {
+                assemble(nextMap, nextHostname, out);
+            }
+        }
+        return out;
+    };
+    return assemble(rootMap, '', []);
+}
+
+/*******************************************************************************
+ * 
+ * For large rulesets, one rule per line for compromise between size and
+ * readability. This also means that the number of lines in resulting file
+ * representative of the number of rules in the ruleset.
+ * 
+ * */
+
+function toJSONRuleset(ruleset) {
     const replacer = (k, v) => {
         if ( k.startsWith('_') ) { return; }
         if ( Array.isArray(v) ) {
@@ -260,13 +318,55 @@ async function processNetworkFilters(assetDetails, network) {
         }
         return v;
     };
+    const indent = ruleset.length > 10 ? undefined : 1;
+    const out = [];
+    for ( const rule of ruleset ) {
+        out.push(JSON.stringify(rule, replacer, indent));
+    }
+    return `[\n${out.join(',\n')}\n]\n`;
+}
 
+/******************************************************************************/
+
+async function processNetworkFilters(assetDetails, network) {
     const { ruleset: rules } = network;
     log(`Input filter count: ${network.filterCount}`);
     log(`\tAccepted filter count: ${network.acceptedFilterCount}`);
     log(`\tRejected filter count: ${network.rejectedFilterCount}`);
     log(`Output rule count: ${rules.length}`);
 
+    // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/declarativeNetRequest/RuleCondition#browser_compatibility
+    // isUrlFilterCaseSensitive is true by default in Chromium. It will be
+    // false by default in Chromium 118+.
+    if ( platform !== 'firefox' ) {
+        for ( const rule of rules ) {
+            const { condition } = rule;
+            if ( condition === undefined ) { continue; }
+            if ( condition.urlFilter === undefined ) {
+                if ( condition.regexFilter === undefined ) { continue; }
+            }
+            if ( condition.isUrlFilterCaseSensitive === undefined ) {
+                condition.isUrlFilterCaseSensitive = false;
+            } else if ( condition.isUrlFilterCaseSensitive === true ) {
+                condition.isUrlFilterCaseSensitive = undefined;
+            }
+        }
+    }
+
+    // Minimize requestDomains arrays
+    for ( const rule of rules ) {
+        const condition = rule.condition;
+        if ( condition === undefined ) { continue; }
+        const requestDomains = condition.requestDomains;
+        if ( requestDomains === undefined ) { continue; }
+        const beforeCount = requestDomains.length;
+        condition.requestDomains = pruneHostnameArray(requestDomains);
+        const afterCount = condition.requestDomains.length;
+        if ( afterCount !== beforeCount ) {
+            log(`\tPruning requestDomains: from ${beforeCount} to ${afterCount}`);
+        }
+    }
+
     const plainGood = rules.filter(rule => isGood(rule) && isRegex(rule) === false);
     log(`\tPlain good: ${plainGood.length}`);
     log(plainGood
@@ -312,34 +412,34 @@ async function processNetworkFilters(assetDetails, network) {
 
     writeFile(
         `${rulesetDir}/main/${assetDetails.id}.json`,
-        `${JSON.stringify(plainGood, replacer, 1)}\n`
+        toJSONRuleset(plainGood)
     );
 
     if ( regexes.length !== 0 ) {
         writeFile(
             `${rulesetDir}/regex/${assetDetails.id}.json`,
-            `${JSON.stringify(regexes, replacer, 1)}\n`
+            toJSONRuleset(regexes)
         );
     }
 
     if ( removeparamsGood.length !== 0 ) {
         writeFile(
             `${rulesetDir}/removeparam/${assetDetails.id}.json`,
-            `${JSON.stringify(removeparamsGood, replacer, 1)}\n`
+            toJSONRuleset(removeparamsGood)
         );
     }
 
     if ( redirects.length !== 0 ) {
         writeFile(
             `${rulesetDir}/redirect/${assetDetails.id}.json`,
-            `${JSON.stringify(redirects, replacer, 1)}\n`
+            toJSONRuleset(redirects)
         );
     }
 
     if ( modifyHeaders.length !== 0 ) {
         writeFile(
             `${rulesetDir}/modify-headers/${assetDetails.id}.json`,
-            `${JSON.stringify(modifyHeaders, replacer, 1)}\n`
+            toJSONRuleset(modifyHeaders)
         );
     }
 
@@ -1013,23 +1113,15 @@ async function rulesetFromURLs(assetDetails) {
 
 async function main() {
 
-    // Get manifest content
-    const manifest = await fs.readFile(
-        `${outputDir}/manifest.json`,
-        { encoding: 'utf8' }
-    ).then(text =>
-        JSON.parse(text)
-    );
-
-    // Create unique version number according to build time
-    let version = manifest.version;
+    let version = '';
     {
         const now = new Date();
-        const yearPart = now.getUTCFullYear() - 2000;
-        const monthPart = (now.getUTCMonth() + 1) * 1000;
-        const dayPart = now.getUTCDate() * 10;
-        const hourPart = Math.floor(now.getUTCHours() / 3) + 1;
-        version += `.${yearPart}.${monthPart + dayPart + hourPart}`;
+        const yearPart = now.getUTCFullYear();
+        const monthPart = now.getUTCMonth() + 1;
+        const dayPart = now.getUTCDate();
+        const hourPart = Math.floor(now.getUTCHours());
+        const minutePart = Math.floor(now.getUTCMinutes());
+        version = `${yearPart}.${monthPart}.${dayPart}.${hourPart * 60 + minutePart}`;
     }
     log(`Version: ${version}`);
 
@@ -1215,6 +1307,13 @@ async function main() {
     await Promise.all(writeOps);
 
     // Patch manifest
+    // Get manifest content
+    const manifest = await fs.readFile(
+        `${outputDir}/manifest.json`,
+        { encoding: 'utf8' }
+    ).then(text =>
+        JSON.parse(text)
+    );
     // Patch declarative_net_request key
     manifest.declarative_net_request = { rule_resources: ruleResources };
     // Patch web_accessible_resources key
@@ -1222,18 +1321,13 @@ async function main() {
         resources: Array.from(requiredRedirectResources).map(path => `/${path}`),
         matches: [ '<all_urls>' ],
     };
-    if ( commandLineArgs.get('platform') === 'chromium' ) {
+    if ( platform === 'chromium' ) {
         web_accessible_resources.use_dynamic_url = true;
     }
     manifest.web_accessible_resources = [ web_accessible_resources ];
 
-    // Patch version key
-    const now = new Date();
-    const yearPart = now.getUTCFullYear() - 2000;
-    const monthPart = (now.getUTCMonth() + 1) * 1000;
-    const dayPart = now.getUTCDate() * 10;
-    const hourPart = Math.floor(now.getUTCHours() / 3) + 1;
-    manifest.version = manifest.version + `.${yearPart}.${monthPart + dayPart + hourPart}`;
+    // Patch manifest version property
+    manifest.version = version;
     // Commit changes
     await fs.writeFile(
         `${outputDir}/manifest.json`,
@@ -1242,7 +1336,6 @@ async function main() {
 
     // Log results
     const logContent = stdOutput.join('\n') + '\n';
-    await fs.writeFile(`${outputDir}/log.txt`, logContent);
     await fs.writeFile(`${cacheDir}/log.txt`, logContent);
 }
 
