@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2017-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -26,8 +26,8 @@
 import logger from './logger.js';
 import µb from './background.js';
 import { sessionFirewall } from './filtering-engines.js';
-
 import { StaticExtFilteringHostnameDB } from './static-ext-filtering-db.js';
+import { entityFromDomain } from './uri-utils.js';
 
 /******************************************************************************/
 
@@ -300,6 +300,14 @@ function applyCSSSelector(details, selector) {
     return modified;
 }
 
+function logError(writer, msg) {
+    logger.writeOne({
+        realm: 'message',
+        type: 'error',
+        text: msg.replace('{who}', writer.properties.get('name') || '?')
+    });
+}
+
 htmlFilteringEngine.reset = function() {
     filterDB.clear();
     pselectors.clear();
@@ -314,41 +322,44 @@ htmlFilteringEngine.freeze = function() {
 };
 
 htmlFilteringEngine.compile = function(parser, writer) {
-    const { raw, compiled, exception } = parser.result;
+    const isException = parser.isException();
+    const { raw, compiled } = parser.result;
     if ( compiled === undefined ) {
-        const who = writer.properties.get('name') || '?';
-        logger.writeOne({
-            realm: 'message',
-            type: 'error',
-            text: `Invalid HTML filter in ${who}: ##${raw}`
-        });
-        return;
+        return logError(writer, `Invalid HTML filter in {who}: ##${raw}`);
     }
 
     writer.select('HTML_FILTERS');
 
     // Only exception filters are allowed to be global.
     if ( parser.hasOptions() === false ) {
-        if ( exception ) {
+        if ( isException ) {
             writer.push([ 64, '', 1, compiled ]);
         }
         return;
     }
 
-    // TODO: Mind negated hostnames, they are currently discarded.
-
-    for ( const { hn, not, bad } of parser.extOptions() ) {
+    const compiledFilters = [];
+    let hasOnlyNegated = true;
+    for ( const { hn, not, bad } of parser.getExtFilterDomainIterator() ) {
         if ( bad ) { continue; }
-        let kind = 0;
-        if ( exception ) {
-            if ( not ) { continue; }
-            kind |= 0b01;
+        let kind = isException ? 0b01 : 0b00;
+        if ( not ) {
+            kind ^= 0b01;
+        } else {
+            hasOnlyNegated = false;
         }
         if ( compiled.charCodeAt(0) === 0x7B /* '{' */ ) {
             kind |= 0b10;
         }
-        writer.push([ 64, hn, kind, compiled ]);
+        compiledFilters.push([ 64, hn, kind, compiled ]);
     }
+
+    // Not allowed since it's equivalent to forbidden generic HTML filters
+    if ( isException === false && hasOnlyNegated ) {
+        return logError(writer, `Invalid HTML filter in {who}: ##${raw}`);
+    }
+
+    writer.pushMany(compiledFilters);
 };
 
 htmlFilteringEngine.fromCompiledContent = function(reader) {
@@ -370,25 +381,21 @@ htmlFilteringEngine.fromCompiledContent = function(reader) {
     }
 };
 
-htmlFilteringEngine.retrieve = function(details) {
-    const hostname = details.hostname;
-
+htmlFilteringEngine.retrieve = function(fctxt) {
     const plains = new Set();
     const procedurals = new Set();
     const exceptions = new Set();
+    const retrieveSets = [ plains, exceptions, procedurals, exceptions ];
 
-    filterDB.retrieve(
-        hostname,
-        [ plains, exceptions, procedurals, exceptions ]
-    );
-    const entity = details.entity !== ''
-        ? `${hostname.slice(0, -details.domain.length)}${details.entity}`
+    const hostname = fctxt.getHostname();
+    filterDB.retrieve(hostname, retrieveSets);
+
+    const domain = fctxt.getDomain();
+    const entity = entityFromDomain(domain);
+    const hostnameEntity = entity !== ''
+        ? `${hostname.slice(0, -domain.length)}${entity}`
         : '*';
-    filterDB.retrieve(
-        entity,
-        [ plains, exceptions, procedurals, exceptions ],
-        1
-    );
+    filterDB.retrieve(hostnameEntity, retrieveSets, 1);
 
     if ( plains.size === 0 && procedurals.size === 0 ) { return; }
 
@@ -410,12 +417,12 @@ htmlFilteringEngine.retrieve = function(details) {
     for ( const selector of exceptions ) {
         if ( plains.has(selector) ) {
             plains.delete(selector);
-            logOne(details, 1, selector);
+            logOne(fctxt, 1, selector);
             continue;
         }
         if ( procedurals.has(selector) ) {
             procedurals.delete(selector);
-            logOne(details, 1, JSON.parse(selector).raw);
+            logOne(fctxt, 1, JSON.parse(selector).raw);
             continue;
         }
     }
@@ -425,15 +432,15 @@ htmlFilteringEngine.retrieve = function(details) {
     }
 };
 
-htmlFilteringEngine.apply = function(doc, details) {
+htmlFilteringEngine.apply = function(doc, details, selectors) {
     docRegister = doc;
     let modified = false;
-    for ( const selector of details.selectors.plains ) {
+    for ( const selector of selectors.plains ) {
         if ( applyCSSSelector(details, selector) ) {
             modified = true;
         }
     }
-    for ( const selector of details.selectors.procedurals ) {
+    for ( const selector of selectors.procedurals ) {
         if ( applyProceduralSelector(details, selector) ) {
             modified = true;
         }
