@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -255,10 +255,9 @@ if ( isBackgroundProcess !== true ) {
         }
 
         for ( const elem of root.querySelectorAll('[placeholder]') ) {
-            elem.setAttribute(
-                'placeholder',
-                i18n$(elem.getAttribute('placeholder'))
-            );
+            const text = i18n$(elem.getAttribute('placeholder'));
+            if ( text === '' ) { continue; }
+            elem.setAttribute('placeholder', text);
         }
 
         for ( const elem of root.querySelectorAll('[data-i18n-tip]') ) {
@@ -294,6 +293,51 @@ if ( isBackgroundProcess !== true ) {
         return i18n$('elapsedManyDaysAgo').replace('{{value}}', Math.floor(value).toLocaleString());
     };
 
+    const unicodeFlagToImageSrc = new Map([
+        [ '🇦🇱', 'al' ], [ '🇦🇷', 'ar' ], [ '🇦🇹', 'at' ], [ '🇧🇦', 'ba' ],
+        [ '🇧🇬', 'bg' ], [ '🇧🇷', 'br' ], [ '🇨🇦', 'ca' ], [ '🇨🇭', 'ch' ],
+        [ '🇨🇳', 'cn' ], [ '🇨🇴', 'co' ], [ '🇨🇾', 'cy' ], [ '🇨🇿', 'cz' ],
+        [ '🇩🇪', 'de' ], [ '🇩🇰', 'dk' ], [ '🇩🇿', 'dz' ], [ '🇪🇪', 'ee' ],
+        [ '🇪🇬', 'eg' ], [ '🇪🇸', 'es' ], [ '🇫🇮', 'fi' ], [ '🇫🇴', 'fo' ],
+        [ '🇫🇷', 'fr' ], [ '🇬🇷', 'gr' ], [ '🇭🇷', 'hr' ], [ '🇭🇺', 'hu' ],
+        [ '🇮🇩', 'id' ], [ '🇮🇱', 'il' ], [ '🇮🇳', 'in' ], [ '🇮🇷', 'ir' ],
+        [ '🇮🇸', 'is' ], [ '🇮🇹', 'it' ], [ '🇯🇵', 'jp' ], [ '🇰🇷', 'kr' ],
+        [ '🇰🇿', 'kz' ], [ '🇱🇰', 'lk' ], [ '🇱🇹', 'lt' ], [ '🇱🇻', 'lv' ],
+        [ '🇲🇦', 'ma' ], [ '🇲🇩', 'md' ], [ '🇲🇰', 'mk' ], [ '🇲🇽', 'mx' ],
+        [ '🇲🇾', 'my' ], [ '🇳🇱', 'nl' ], [ '🇳🇴', 'no' ], [ '🇳🇵', 'np' ],
+        [ '🇵🇱', 'pl' ], [ '🇵🇹', 'pt' ], [ '🇷🇴', 'ro' ], [ '🇷🇸', 'rs' ],
+        [ '🇷🇺', 'ru' ], [ '🇸🇦', 'sa' ], [ '🇸🇮', 'si' ], [ '🇸🇰', 'sk' ],
+        [ '🇸🇪', 'se' ], [ '🇸🇷', 'sr' ], [ '🇹🇭', 'th' ], [ '🇹🇯', 'tj' ],
+        [ '🇹🇼', 'tw' ], [ '🇹🇷', 'tr' ], [ '🇺🇦', 'ua' ], [ '🇺🇿', 'uz' ],
+        [ '🇻🇳', 'vn' ], [ '🇽🇰', 'xk' ],
+    ]);
+    const reUnicodeFlags = new RegExp(
+        Array.from(unicodeFlagToImageSrc).map(a => a[0]).join('|'),
+        'gu'
+    );
+    i18n.patchUnicodeFlags = function(text) {
+        const fragment = document.createDocumentFragment();
+        let i = 0;
+        for (;;) {
+            const match = reUnicodeFlags.exec(text);
+            if ( match === null ) { break; }
+            if ( match.index > i ) {
+                fragment.append(text.slice(i, match.index));
+            }
+            const img = document.createElement('img');
+            const countryCode = unicodeFlagToImageSrc.get(match[0]);
+            img.src = `/img/flags-of-the-world/${countryCode}.png`;
+            img.title = countryCode;
+            img.classList.add('countryFlag');
+            fragment.append(img, '\u200A');
+            i = reUnicodeFlags.lastIndex;
+        }
+        if ( i < text.length ) {
+            fragment.append(text.slice(i));
+        }
+        return fragment; 
+    };
+
     i18n.render();
 }
 
