@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2015-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -26,8 +26,8 @@
 import staticNetFilteringEngine from './static-net-filtering.js';
 import µb from './background.js';
 import { CompiledListWriter } from './static-filtering-io.js';
-import { StaticFilteringParser } from './static-filtering-parser.js';
 import { i18n$ } from './i18n.js';
+import * as sfp from './static-filtering-parser.js';
 
 import {
     domainFromHostname,
@@ -36,11 +36,9 @@ import {
 
 /******************************************************************************/
 
-const workerTTL = 5 * 60 * 1000;
 const pendingResponses = new Map();
 
 let worker = null;
-let workerTTLTimer;
 let needLists = true;
 let messageId = 1;
 
@@ -52,10 +50,7 @@ const onWorkerMessage = function(e) {
 };
 
 const stopWorker = function() {
-    if ( workerTTLTimer !== undefined ) {
-        clearTimeout(workerTTLTimer);
-        workerTTLTimer = undefined;
-    }
+    workerTTLTimer.off();
     if ( worker === null ) { return; }
     worker.terminate();
     worker = null;
@@ -66,6 +61,9 @@ const stopWorker = function() {
     pendingResponses.clear();
 };
 
+const workerTTLTimer = vAPI.defer.create(stopWorker);
+const workerTTL = { min: 5 };
+
 const initWorker = function() {
     if ( worker === null ) {
         worker = new Worker('js/reverselookup-worker.js');
@@ -73,10 +71,7 @@ const initWorker = function() {
     }
 
     // The worker will be shutdown after n minutes without being used.
-    if ( workerTTLTimer !== undefined ) {
-        clearTimeout(workerTTLTimer);
-    }
-    workerTTLTimer = vAPI.setTimeout(stopWorker, workerTTL);
+    workerTTLTimer.offon(workerTTL);
 
     if ( needLists === false ) {
         return Promise.resolve();
@@ -134,23 +129,24 @@ const fromNetFilter = async function(rawFilter) {
     if ( typeof rawFilter !== 'string' || rawFilter === '' ) { return; }
 
     const writer = new CompiledListWriter();
-    const parser = new StaticFilteringParser({
+    const parser = new sfp.AstFilterParser({
+        trustedSource: true,
+        maxTokenLength: staticNetFilteringEngine.MAX_TOKEN_LENGTH,
         nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
     });
-    parser.setMaxTokenLength(staticNetFilteringEngine.MAX_TOKEN_LENGTH);
-    parser.analyze(rawFilter);
+    parser.parse(rawFilter);
 
-    const compiler = staticNetFilteringEngine.createCompiler(parser);
-    if ( compiler.compile(writer) === false ) { return; }
+    const compiler = staticNetFilteringEngine.createCompiler();
+    if ( compiler.compile(parser, writer) === false ) { return; }
 
     await initWorker();
 
     const id = messageId++;
     worker.postMessage({
         what: 'fromNetFilter',
-        id: id,
+        id,
         compiledFilter: writer.last(),
-        rawFilter: rawFilter
+        rawFilter,
     });
 
     return new Promise(resolve => {
@@ -171,11 +167,21 @@ const fromExtendedFilter = async function(details) {
     const id = messageId++;
     const hostname = hostnameFromURI(details.url);
 
+    const parser = new sfp.AstFilterParser({
+        trustedSource: true,
+        nativeCssHas: vAPI.webextFlavor.env.includes('native_css_has'),
+    });
+    parser.parse(details.rawFilter);
+    let compiled;
+    if ( parser.isScriptletFilter() ) {
+        compiled = JSON.stringify(parser.getScriptletArgs());
+    }
+
     worker.postMessage({
         what: 'fromExtendedFilter',
-        id: id,
+        id,
         domain: domainFromHostname(hostname),
-        hostname: hostname,
+        hostname,
         ignoreGeneric:
             staticNetFilteringEngine.matchRequestReverse(
                 'generichide',
@@ -186,7 +192,8 @@ const fromExtendedFilter = async function(details) {
                 'specifichide',
                 details.url
             ) === 2,
-        rawFilter: details.rawFilter
+        rawFilter: details.rawFilter,
+        compiled,
     });
 
     return new Promise(resolve => {
