@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2015-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -19,6 +19,8 @@
     Home: https://github.com/gorhill/uBlock
 */
 
+/* globals browser */
+
 'use strict';
 
 /******************************************************************************/
@@ -28,12 +30,8 @@
 
 /******************************************************************************/
 
-if (
-    typeof vAPI !== 'object' ||
-    vAPI.domWatcher instanceof Object === false
-) {
-    return;
-}
+if ( typeof vAPI !== 'object' ) { return; }
+if ( vAPI.domWatcher instanceof Object === false ) { return; }
 
 const reHasCSSCombinators = /[ >+~]/;
 const simpleDeclarativeSet = new Set();
@@ -51,16 +49,16 @@ const loggedSelectors = new Set();
 
 const rePseudoElements = /:(?::?after|:?before|:[a-z-]+)$/;
 
-const hasSelector = function(selector, context = document) {
+function hasSelector(selector, context = document) {
     try {
         return context.querySelector(selector) !== null;
     }
     catch(ex) {
     }
     return false;
-};
+}
 
-const safeMatchSelector = function(selector, context) {
+function safeMatchSelector(selector, context) {
     const safeSelector = rePseudoElements.test(selector)
         ? selector.replace(rePseudoElements, '')
         : selector;
@@ -70,9 +68,9 @@ const safeMatchSelector = function(selector, context) {
     catch(ex) {
     }
     return false;
-};
+}
 
-const safeQuerySelector = function(selector, context = document) {
+function safeQuerySelector(selector, context = document) {
     const safeSelector = rePseudoElements.test(selector)
         ? selector.replace(rePseudoElements, '')
         : selector;
@@ -82,9 +80,9 @@ const safeQuerySelector = function(selector, context = document) {
     catch(ex) {
     }
     return null;
-};
+}
 
-const safeGroupSelectors = function(selectors) {
+function safeGroupSelectors(selectors) {
     const arr = Array.isArray(selectors)
         ? selectors
         : Array.from(selectors);
@@ -93,11 +91,11 @@ const safeGroupSelectors = function(selectors) {
             ? s.replace(rePseudoElements, '')
             : s;
     }).join(',\n');
-};
+}
 
 /******************************************************************************/
 
-const processDeclarativeSimple = function(node, out) {
+function processDeclarativeSimple(node, out) {
     if ( simpleDeclarativeSet.size === 0 ) { return; }
     if ( simpleDeclarativeStr === undefined ) {
         simpleDeclarativeStr = safeGroupSelectors(simpleDeclarativeSet);
@@ -120,11 +118,11 @@ const processDeclarativeSimple = function(node, out) {
         simpleDeclarativeStr = undefined;
         loggedSelectors.add(selector);
     }
-};
+}
 
 /******************************************************************************/
 
-const processDeclarativeComplex = function(out) {
+function processDeclarativeComplex(out) {
     if ( complexDeclarativeSet.size === 0 ) { return; }
     if ( complexDeclarativeStr === undefined ) {
         complexDeclarativeStr = safeGroupSelectors(complexDeclarativeSet);
@@ -137,24 +135,26 @@ const processDeclarativeComplex = function(out) {
         complexDeclarativeStr = undefined;
         loggedSelectors.add(selector);
     }
-};
+}
 
 /******************************************************************************/
 
-const processProcedural = function(out) {
+function processProcedural(out) {
     if ( proceduralDict.size === 0 ) { return; }
     for ( const [ raw, pselector ] of proceduralDict ) {
-        if ( pselector.hit === false && pselector.exec().length === 0 ) {
+        if ( pselector.converted ) {
+            if ( safeQuerySelector(pselector.selector) === null ) { continue; }
+        } else if ( pselector.hit === false && pselector.exec().length === 0 ) {
             continue;
         }
         out.push(`##${raw}`);
         proceduralDict.delete(raw);
     }
-};
+}
 
 /******************************************************************************/
 
-const processExceptions = function(out) {
+function processExceptions(out) {
     if ( exceptionDict.size === 0 ) { return; }
     if ( exceptionStr === undefined ) {
         exceptionStr = safeGroupSelectors(exceptionDict.keys());
@@ -167,18 +167,18 @@ const processExceptions = function(out) {
         exceptionStr = undefined;
         loggedSelectors.add(raw);
     }
-};
+}
 
 /******************************************************************************/
 
-const processProceduralExceptions = function(out) {
+function processProceduralExceptions(out) {
     if ( proceduralExceptionDict.size === 0 ) { return; }
     for ( const exception of proceduralExceptionDict.values() ) {
         if ( exception.test() === false ) { continue; }
         out.push(`#@#${exception.raw}`);
         proceduralExceptionDict.delete(exception.raw);
     }
-};
+}
 
 /******************************************************************************/
 
@@ -269,7 +269,7 @@ const handlers = {
                 if (
                     details.action !== undefined &&
                     details.tasks === undefined &&
-                    details.action[0] === ':style'
+                    details.action[0] === 'style'
                 ) {
                     exceptionDict.set(details.selector, details.raw);
                     continue;
@@ -311,9 +311,22 @@ const handlers = {
     }
 };
 
+vAPI.domWatcher.addListener(handlers);
+
 /******************************************************************************/
 
-const shutdown = function() {
+const broadcastHandler = msg => {
+    if ( msg.what === 'loggerDisabled' ) {
+        shutdown();
+    }
+};
+
+browser.runtime.onMessage.addListener(broadcastHandler);
+
+/******************************************************************************/
+
+function shutdown() {
+    browser.runtime.onMessage.removeListener(broadcastHandler);
     processTimer.clear();
     attributeObserver.disconnect();
     if ( typeof vAPI !== 'object' ) { return; }
@@ -323,29 +336,7 @@ const shutdown = function() {
     if ( vAPI.domWatcher instanceof Object ) {
         vAPI.domWatcher.removeListener(handlers);
     }
-    if ( vAPI.broadcastListener instanceof Object ) {
-        vAPI.broadcastListener.remove(broadcastListener);
-    }
-};
-
-/******************************************************************************/
-
-const broadcastListener = msg => {
-    if ( msg.what === 'loggerDisabled' ) {
-        shutdown();
-    }
-};
-
-/******************************************************************************/
-
-vAPI.messaging.extend().then(extended => {
-    if ( extended !== true ) {
-        return shutdown();
-    }
-    vAPI.broadcastListener.add(broadcastListener);
-});
-
-vAPI.domWatcher.addListener(handlers);
+}
 
 /******************************************************************************/
 
