@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -32,6 +32,9 @@ import scriptletFilteringEngine from './scriptlet-filtering.js';
 import staticNetFilteringEngine from './static-net-filtering.js';
 import textEncode from './text-encode.js';
 import µb from './background.js';
+import * as sfp from './static-filtering-parser.js';
+import * as fc from  './filtering-context.js';
+import { isNetworkURI } from './uri-utils.js';
 
 import {
     sessionFirewall,
@@ -39,10 +42,6 @@ import {
     sessionURLFiltering,
 } from './filtering-engines.js';
 
-import {
-    entityFromDomain,
-    isNetworkURI,
-} from './uri-utils.js';
 
 /******************************************************************************/
 
@@ -61,9 +60,6 @@ window.addEventListener('webextFlavor', function() {
         vAPI.webextFlavor.soup.has('firefox');
 }, { once: true });
 
-// https://github.com/uBlockOrigin/uBlock-issues/issues/1553
-const supportsFloc = document.interestCohort instanceof Function;
-
 /******************************************************************************/
 
 const patchLocalRedirectURL = url => url.charCodeAt(0) === 0x2F /* '/' */
@@ -203,12 +199,7 @@ const onBeforeRootFrameRequest = function(fctxt) {
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/760
     //   Redirect non-blocked request?
-    if (
-        result !== 1 &&
-        trusted === false &&
-        pageStore !== null &&
-        staticNetFilteringEngine.hasQuery(fctxt)
-    ) {
+    if ( result !== 1 && trusted === false && pageStore !== null ) {
         pageStore.redirectNonBlockedRequest(fctxt);
     }
 
@@ -442,23 +433,23 @@ const onBeforeBehindTheSceneRequest = function(fctxt) {
 //   request.
 
 {
+    const pageStores = new Set();
     let hostname = '';
-    let pageStores = new Set();
     let pageStoresToken = 0;
-    let gcTimer;
 
     const reset = function() {
         hostname = '';
-        pageStores = new Set();
+        pageStores.clear();
         pageStoresToken = 0;
     };
 
     const gc = ( ) => {
-        gcTimer = undefined;
         if ( pageStoresToken !== µb.pageStoresToken ) { return reset(); }
-        gcTimer = vAPI.setTimeout(gc, 30011);
+        gcTimer.on(30011);
     };
 
+    const gcTimer = vAPI.defer.create(gc);
+
     onBeforeBehindTheSceneRequest.journalAddRequest = (fctxt, result) => {
         const docHostname = fctxt.getDocHostname();
         if (
@@ -466,16 +457,13 @@ const onBeforeBehindTheSceneRequest = function(fctxt) {
             pageStoresToken !== µb.pageStoresToken
         ) {
             hostname = docHostname;
-            pageStores = new Set();
+            pageStores.clear();
             for ( const pageStore of µb.pageStores.values() ) {
                 if ( pageStore.tabHostname !== docHostname ) { continue; }
                 pageStores.add(pageStore);
             }
             pageStoresToken = µb.pageStoresToken;
-            if ( gcTimer !== undefined ) {
-                clearTimeout(gcTimer);
-            }
-            gcTimer = vAPI.setTimeout(gc, 30011);
+            gcTimer.offon(30011);
         }
         for ( const pageStore of pageStores ) {
             pageStore.journalAddRequest(fctxt, result);
@@ -492,14 +480,6 @@ const onBeforeBehindTheSceneRequest = function(fctxt) {
 // - CSP injection
 
 const onHeadersReceived = function(details) {
-    // https://github.com/uBlockOrigin/uBlock-issues/issues/610
-    //   Process behind-the-scene requests in a special way.
-    if (
-        details.tabId < 0 &&
-        normalizeBehindTheSceneResponseHeaders(details) === false
-    ) {
-        return;
-    }
 
     const fctxt = µb.filteringContext.fromWebrequestDetails(details);
     const isRootDoc = fctxt.itype === fctxt.MAIN_FRAME;
@@ -522,7 +502,7 @@ const onHeadersReceived = function(details) {
     const { responseHeaders } = details;
     if ( Array.isArray(responseHeaders) === false ) { return; }
 
-    if ( isRootDoc === false && µb.hiddenSettings.filterOnHeaders === true ) {
+    if ( isRootDoc === false ) {
         const result = pageStore.filterOnHeaders(fctxt, responseHeaders);
         if ( result !== 0 ) {
             if ( logger.enabled ) {
@@ -535,25 +515,45 @@ const onHeadersReceived = function(details) {
         }
     }
 
-    if ( isRootDoc === false && fctxt.itype !== fctxt.SUB_FRAME ) { return; }
+    const mime = mimeFromHeaders(responseHeaders);
 
     // https://github.com/gorhill/uBlock/issues/2813
     //   Disable the blocking of large media elements if the document is itself
     //   a media element: the resource was not prevented from loading so no
     //   point to further block large media elements for the current document.
     if ( isRootDoc ) {
-        const contentType = headerValueFromName('content-type', responseHeaders);
-        if ( reMediaContentTypes.test(contentType) ) {
+        if ( reMediaContentTypes.test(mime) ) {
             pageStore.allowLargeMediaElementsUntil = 0;
             // Fall-through: this could be an SVG document, which supports
             // script tags.
         }
     }
 
-    // At this point we have a HTML document.
-
-    const filteredHTML =
-        µb.canFilterResponseData && filterDocument(fctxt, details) === true;
+    if ( bodyFilterer.canFilter(fctxt, details) ) {
+        const jobs = [];
+        // `replace=` filter option
+        const replaceDirectives =
+            staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'replace');
+        if ( replaceDirectives ) {
+            jobs.push({
+                fn: textResponseFilterer,
+                args: [ replaceDirectives ],
+            });
+        }
+        // html filtering
+        if ( mime === 'text/html' || mime === 'application/xhtml+xml' ) {
+            const selectors = htmlFilteringEngine.retrieve(fctxt);
+            if ( selectors ) {
+                jobs.push({
+                    fn: htmlResponseFilterer,
+                    args: [ selectors ],
+                });
+            }
+        }
+        if ( jobs.length !== 0 ) {
+            bodyFilterer.doFilter(fctxt, jobs);
+        }
+    }
 
     let modifiedHeaders = false;
     if ( httpheaderFilteringEngine.apply(fctxt, responseHeaders) === true ) {
@@ -562,7 +562,7 @@ const onHeadersReceived = function(details) {
     if ( injectCSP(fctxt, pageStore, responseHeaders) === true ) {
         modifiedHeaders = true;
     }
-    if ( supportsFloc && foilFloc(fctxt, responseHeaders) ) {
+    if ( injectPP(fctxt, pageStore, responseHeaders) === true ) {
         modifiedHeaders = true;
     }
 
@@ -572,7 +572,7 @@ const onHeadersReceived = function(details) {
     // https://github.com/uBlockOrigin/uBlock-issues/issues/229
     //   Use `no-cache` instead of `no-cache, no-store, must-revalidate`, this
     //   allows Firefox's offline mode to work as expected.
-    if ( (filteredHTML || modifiedHeaders) && dontCacheResponseHeaders ) {
+    if ( modifiedHeaders && dontCacheResponseHeaders ) {
         const cacheControl = µb.hiddenSettings.cacheControlForFirefox1376932;
         if ( cacheControl !== 'unset' ) {
             let i = headerIndexFromName('cache-control', responseHeaders);
@@ -594,288 +594,353 @@ const reMediaContentTypes = /^(?:audio|image|video)\//;
 
 /******************************************************************************/
 
-// https://github.com/uBlockOrigin/uBlock-issues/issues/610
-
-const normalizeBehindTheSceneResponseHeaders = function(details) {
-    if ( details.type !== 'xmlhttprequest' ) { return false; }
-    const headers = details.responseHeaders;
-    if ( Array.isArray(headers) === false ) { return false; }
-    const contentType = headerValueFromName('content-type', headers);
-    if ( contentType === '' ) { return false; }
-    if ( reMediaContentTypes.test(contentType) === false ) { return false; }
-    if ( contentType.startsWith('image') ) {
-        details.type = 'image';
-    } else {
-        details.type = 'media';
-    }
-    return true;
+const mimeFromHeaders = headers => {
+    if ( Array.isArray(headers) === false ) { return ''; }
+    return mimeFromContentType(headerValueFromName('content-type', headers));
 };
 
+const mimeFromContentType = contentType => {
+    const match = reContentTypeMime.exec(contentType);
+    if ( match === null ) { return ''; }
+    return match[0].toLowerCase();
+};
+
+const reContentTypeMime = /^[^;]+/i;
+
+/******************************************************************************/
+
+function textResponseFilterer(session, directives) {
+    const applied = [];
+    for ( const directive of directives ) {
+        if ( directive.refs instanceof Object === false ) { continue; }
+        if ( directive.result !== 1 ) {
+            applied.push(directive);
+            continue;
+        }
+        const { refs } = directive;
+        if ( refs.$cache === null ) {
+            refs.$cache = sfp.parseReplaceValue(refs.value);
+        }
+        const cache = refs.$cache;
+        if ( cache === undefined ) { continue; }
+        cache.re.lastIndex = 0;
+        if ( cache.re.test(session.getString()) !== true ) { continue; }
+        cache.re.lastIndex = 0;
+        session.setString(session.getString().replace(
+            cache.re,
+            cache.replacement
+        ));
+        applied.push(directive);
+    }
+    if ( applied.length === 0 ) { return; }
+    if ( logger.enabled !== true ) { return; }
+    session.setRealm('network')
+         .pushFilters(applied.map(a => a.logData()))
+         .toLogger();
+}
+
+/******************************************************************************/
+
+function htmlResponseFilterer(session, selectors) {
+    if ( htmlResponseFilterer.domParser === null ) {
+        htmlResponseFilterer.domParser = new DOMParser();
+        htmlResponseFilterer.xmlSerializer = new XMLSerializer();
+    }
+
+    const doc = htmlResponseFilterer.domParser.parseFromString(
+        session.getString(),
+        session.mime
+    );
+
+    if ( selectors === undefined ) { return; }
+    if ( htmlFilteringEngine.apply(doc, session, selectors) !== true ) { return; }
+
+    // https://stackoverflow.com/questions/6088972/get-doctype-of-an-html-as-string-with-javascript/10162353#10162353
+    const doctypeStr = [
+        doc.doctype instanceof Object ?
+            htmlResponseFilterer.xmlSerializer.serializeToString(doc.doctype) + '\n' :
+            '',
+        doc.documentElement.outerHTML,
+    ].join('\n');
+    session.setString(doctypeStr);
+}
+htmlResponseFilterer.domParser = null;
+htmlResponseFilterer.xmlSerializer = null;
+
+
 /*******************************************************************************
 
     The response body filterer is responsible for:
 
+    - Realize static network filter option `replace=`
     - HTML filtering
 
-    In the spirit of efficiency, the response body filterer works this way:
-
-    If:
-        - HTML filtering: no.
-    Then:
-        No response body filtering is initiated.
-
-    If:
-        - HTML filtering: yes.
-    Then:
-        Assemble all response body data into a single buffer. Once all the
-        response data has been received, create a document from it. Then:
-        - Remove all DOM elements matching HTML filters.
-        Then serialize the resulting modified document as the new response
-        body.
-
 **/
 
-const filterDocument = (( ) => {
-    const filterers = new Map();
-    let domParser, xmlSerializer,
-        utf8TextDecoder, textDecoder, textEncoder;
-
-    const textDecode = function(encoding, buffer) {
-        if (
-            textDecoder !== undefined &&
-            textDecoder.encoding !== encoding
-        ) {
-            textDecoder = undefined;
-        }
-        if ( textDecoder === undefined ) {
-            textDecoder = new TextDecoder(encoding);
-        }
-        return textDecoder.decode(buffer);
-    };
-
-    const reContentTypeDocument = /^(?:text\/html|application\/xhtml\+xml)/i;
+const bodyFilterer = (( ) => {
+    const sessions = new Map();
     const reContentTypeCharset = /charset=['"]?([^'" ]+)/i;
+    const otherValidMimes = new Set([
+        'application/javascript',
+        'application/json',
+        'application/mpegurl',
+        'application/vnd.api+json',
+        'application/vnd.apple.mpegurl',
+        'application/vnd.apple.mpegurl.audio',
+        'application/x-javascript',
+        'application/x-mpegurl',
+        'application/xhtml+xml',
+        'application/xml',
+        'audio/mpegurl',
+        'audio/x-mpegurl',
+    ]);
+    const BINARY_TYPES = fc.FONT | fc.IMAGE | fc.MEDIA | fc.WEBSOCKET;
+    const MAX_BUFFER_LENGTH = 3 * 1024 * 1024;
 
-    const mimeFromContentType = function(contentType) {
-        const match = reContentTypeDocument.exec(contentType);
-        if ( match !== null ) {
-            return match[0].toLowerCase();
+    let textDecoder, textEncoder;
+    let mime = '';
+    let charset = '';
+
+    const contentTypeFromDetails = details => {
+        switch ( details.type ) {
+            case 'script':
+                return 'text/javascript; charset=utf-8';
+            case 'stylesheet':
+                return 'text/css';
+            default:
+                break;
         }
+        return '';
     };
 
-    const charsetFromContentType = function(contentType) {
+    const charsetFromContentType = contentType => {
         const match = reContentTypeCharset.exec(contentType);
-        if ( match !== null ) {
-            return match[1].toLowerCase();
-        }
+        if ( match === null ) { return; }
+        return match[1].toLowerCase();
     };
 
-    const charsetFromDoc = function(doc) {
-        let meta = doc.querySelector('meta[charset]');
-        if ( meta !== null ) {
-            return meta.getAttribute('charset').toLowerCase();
-        }
-        meta = doc.querySelector(
-            'meta[http-equiv="content-type" i][content]'
-        );
-        if ( meta !== null ) {
-            return charsetFromContentType(meta.getAttribute('content'));
+    const charsetFromMime = mime => {
+        switch ( mime ) {
+            case 'application/xml':
+            case 'application/xhtml+xml':
+            case 'text/html':
+            case 'text/css':
+                return;
+            default:
+                break;
         }
+        return 'utf-8';
     };
 
-    const streamClose = function(filterer, buffer) {
+    const charsetFromStream = bytes => {
+        if ( bytes.length < 3 ) { return; }
+        if ( bytes[0] === 0xEF && bytes[1] === 0xBB && bytes[2] === 0xBF ) {
+            return 'utf-8';
+        }
+        let i = -1;
+        while ( i < 65536 ) {
+            i += 1;
+            /* c */ if ( bytes[i+0] !== 0x63 ) { continue; }
+            /* h */ if ( bytes[i+1] !== 0x68 ) { continue; }
+            /* a */ if ( bytes[i+2] !== 0x61 ) { continue; }
+            /* r */ if ( bytes[i+3] !== 0x72 ) { continue; }
+            /* s */ if ( bytes[i+4] !== 0x73 ) { continue; }
+            /* e */ if ( bytes[i+5] !== 0x65 ) { continue; }
+            /* t */ if ( bytes[i+6] !== 0x74 ) { continue; }
+            break;
+        }
+        if ( (i - 40) >= 65536 ) { return; }
+        i += 8;
+        // find first alpha character
+        let j = -1;
+        while ( j < 8 ) {
+            j += 1;
+            const c = bytes[i+j];
+            if ( c >= 0x41 && c <= 0x5A ) { break; }
+            if ( c >= 0x61 && c <= 0x7A ) { break; }
+        }
+        if ( j === 8 ) { return; }
+        i += j;
+        // Collect characters until first non charset-name-character
+        const chars = [];
+        j = 0;
+        while ( j < 24 ) {
+            const c = bytes[i+j];
+            if ( c < 0x2D ) { break; }
+            if ( c > 0x2D && c < 0x30 ) { break; }
+            if ( c > 0x39 && c < 0x41 ) { break; }
+            if ( c > 0x5A && c < 0x61 ) { break; }
+            if ( c > 0x7A ) { break; }
+            chars.push(c);
+            j += 1;
+        }
+        if ( j === 20 ) { return; }
+        return String.fromCharCode(...chars).toLowerCase();
+    };
+
+    const streamClose = (session, buffer) => {
         if ( buffer !== undefined ) {
-            filterer.stream.write(buffer);
-        } else if ( filterer.buffer !== undefined ) {
-            filterer.stream.write(filterer.buffer);
+            session.stream.write(buffer);
+        } else if ( session.buffer !== undefined ) {
+            session.stream.write(session.buffer);
         }
-        filterer.stream.close();
+        session.stream.close();
     };
 
     const onStreamData = function(ev) {
-        const filterer = filterers.get(this);
-        if ( filterer === undefined ) {
+        const session = sessions.get(this);
+        if ( session === undefined ) {
             this.write(ev.data);
             this.disconnect();
             return;
         }
-        if (
-            this.status !== 'transferringdata' &&
-            this.status !== 'finishedtransferringdata'
-        ) {
-            filterers.delete(this);
-            this.disconnect();
-            return;
+        if ( this.status !== 'transferringdata' ) {
+            if ( this.status !== 'finishedtransferringdata' ) {
+                sessions.delete(this);
+                this.disconnect();
+                return;
+            }
         }
-        // TODO:
-        // - Possibly improve buffer growth, if benchmarking shows it's worth
-        //   it.
-        // - Also evaluate whether keeping a list of buffers and then decoding
-        //   them in sequence using TextDecoder's "stream" option is more
-        //   efficient. Can the data buffers be safely kept around for later
-        //   use?
-        // - Informal, quick benchmarks seem to show most of the overhead is
-        //   from calling TextDecoder.decode() and TextEncoder.encode(), and if
-        //   confirmed, there is nothing which can be done uBO-side to reduce
-        //   overhead.
-        if ( filterer.buffer === null ) {
-            filterer.buffer = new Uint8Array(ev.data);
+        if ( session.buffer === null ) {
+            session.buffer = new Uint8Array(ev.data);
             return;
         }
         const buffer = new Uint8Array(
-            filterer.buffer.byteLength +
-            ev.data.byteLength
+            session.buffer.byteLength + ev.data.byteLength
         );
-        buffer.set(filterer.buffer);
-        buffer.set(new Uint8Array(ev.data), filterer.buffer.byteLength);
-        filterer.buffer = buffer;
+        buffer.set(session.buffer);
+        buffer.set(new Uint8Array(ev.data), session.buffer.byteLength);
+        session.buffer = buffer;
+        if ( session.buffer.length >= MAX_BUFFER_LENGTH ) {
+            sessions.delete(this);
+            this.write(session.buffer);
+            this.disconnect();
+        }
     };
 
     const onStreamStop = function() {
-        const filterer = filterers.get(this);
-        filterers.delete(this);
-        if ( filterer === undefined || filterer.buffer === null ) {
+        const session = sessions.get(this);
+        sessions.delete(this);
+        if ( session === undefined || session.buffer === null ) {
             this.close();
             return;
         }
         if ( this.status !== 'finishedtransferringdata' ) { return; }
 
-        if ( domParser === undefined ) {
-            domParser = new DOMParser();
-            xmlSerializer = new XMLSerializer();
+        // If encoding is still unknown, try to extract from stream data
+        if ( session.charset === undefined ) {
+            const charsetFound = charsetFromStream(session.buffer);
+            if ( charsetFound === undefined ) { return streamClose(session); }
+            const charsetUsed = textEncode.normalizeCharset(charsetFound);
+            if ( charsetUsed === undefined ) { return streamClose(session); }
+            session.charset = charsetUsed;
         }
+
+        while ( session.jobs.length !== 0 ) {
+            const job = session.jobs.shift();
+            job.fn(session, ...job.args);
+        }
+        if ( session.modified !== true ) { return streamClose(session); }
+
         if ( textEncoder === undefined ) {
             textEncoder = new TextEncoder();
         }
+        let encodedStream = textEncoder.encode(session.str);
 
-        let doc;
-
-        // If stream encoding is still unknnown, try to extract from document.
-        let charsetFound = filterer.charset,
-            charsetUsed = charsetFound;
-        if ( charsetFound === undefined ) {
-            if ( utf8TextDecoder === undefined ) {
-                utf8TextDecoder = new TextDecoder();
-            }
-            doc = domParser.parseFromString(
-                utf8TextDecoder.decode(filterer.buffer.slice(0, 1024)),
-                filterer.mime
-            );
-            charsetFound = charsetFromDoc(doc);
-            charsetUsed = textEncode.normalizeCharset(charsetFound);
-            if ( charsetUsed === undefined ) {
-                return streamClose(filterer);
-            }
+        if ( session.charset !== 'utf-8' ) {
+            encodedStream = textEncode.encode(session.charset, encodedStream);
         }
 
-        doc = domParser.parseFromString(
-            textDecode(charsetUsed, filterer.buffer),
-            filterer.mime
-        );
-
-        // https://github.com/gorhill/uBlock/issues/3507
-        //   In case of no explicit charset found, try to find one again, but
-        //   this time with the whole document parsed.
-        if ( charsetFound === undefined ) {
-            charsetFound = textEncode.normalizeCharset(charsetFromDoc(doc));
-            if ( charsetFound !== charsetUsed ) {
-                if ( charsetFound === undefined ) {
-                    return streamClose(filterer);
-                }
-                charsetUsed = charsetFound;
-                doc = domParser.parseFromString(
-                    textDecode(charsetFound, filterer.buffer),
-                    filterer.mime
-                );
-            }
-        }
-
-        let modified = false;
-        if ( filterer.selectors !== undefined ) {
-            if ( htmlFilteringEngine.apply(doc, filterer) ) {
-                modified = true;
-            }
-        }
-
-        if ( modified === false ) {
-            return streamClose(filterer);
-        }
-
-        // https://stackoverflow.com/questions/6088972/get-doctype-of-an-html-as-string-with-javascript/10162353#10162353
-        const doctypeStr = doc.doctype instanceof Object ?
-                xmlSerializer.serializeToString(doc.doctype) + '\n' :
-                '';
-
-        // https://github.com/gorhill/uBlock/issues/3391
-        let encodedStream = textEncoder.encode(
-            doctypeStr +
-            doc.documentElement.outerHTML
-        );
-        if ( charsetUsed !== 'utf-8' ) {
-            encodedStream = textEncode.encode(
-                charsetUsed,
-                encodedStream
-            );
-        }
-
-        streamClose(filterer, encodedStream);
+        streamClose(session, encodedStream);
     };
 
     const onStreamError = function() {
-        filterers.delete(this);
+        sessions.delete(this);
     };
 
-    return function(fctxt, extras) {
-        // https://github.com/gorhill/uBlock/issues/3478
-        const statusCode = extras.statusCode || 0;
-        if ( statusCode !== 0 && (statusCode < 200 || statusCode >= 300) ) {
-            return;
+    return class Session extends µb.FilteringContext {
+        constructor(fctxt, mime, charset, jobs) {
+            super(fctxt);
+            this.stream = null;
+            this.buffer = null;
+            this.mime = mime;
+            this.charset = charset;
+            this.str = null;
+            this.modified = false;
+            this.jobs = jobs;
         }
-
-        const hostname = fctxt.getHostname();
-        if ( hostname === '' ) { return; }
-
-        const domain = fctxt.getDomain();
-
-        const request = {
-            stream: undefined,
-            tabId: fctxt.tabId,
-            url: fctxt.url,
-            hostname: hostname,
-            domain: domain,
-            entity: entityFromDomain(domain),
-            selectors: undefined,
-            buffer: null,
-            mime: 'text/html',
-            charset: undefined
-        };
-
-        request.selectors = htmlFilteringEngine.retrieve(request);
-        if ( request.selectors === undefined ) { return; }
-
-        const headers = extras.responseHeaders;
-        const contentType = headerValueFromName('content-type', headers);
-        if ( contentType !== '' ) {
-            request.mime = mimeFromContentType(contentType);
-            if ( request.mime === undefined ) { return; }
-            let charset = charsetFromContentType(contentType);
-            if ( charset !== undefined ) {
-                charset = textEncode.normalizeCharset(charset);
-                if ( charset === undefined ) { return; }
-                request.charset = charset;
+        getString() {
+            if ( this.str !== null ) { return this.str; }
+            if ( textDecoder !== undefined ) {
+                if ( textDecoder.encoding !== this.charset ) {
+                    textDecoder = undefined;
+                }
             }
+            if ( textDecoder === undefined ) {
+                textDecoder = new TextDecoder(this.charset);
+            }
+            this.str = textDecoder.decode(this.buffer);
+            return this.str;
         }
-        // https://bugzilla.mozilla.org/show_bug.cgi?id=1426789
-        if ( headerValueFromName('content-disposition', headers) ) { return; }
+        setString(s) {
+            this.str = s;
+            this.modified = true;
+        }
+        static doFilter(fctxt, jobs) {
+            if ( jobs.length === 0 ) { return; }
+            const session = new Session(fctxt, mime, charset, jobs);
+            session.stream = browser.webRequest.filterResponseData(session.id);
+            session.stream.ondata = onStreamData;
+            session.stream.onstop = onStreamStop;
+            session.stream.onerror = onStreamError;
+            sessions.set(session.stream, session);
+        }
+        static canFilter(fctxt, details) {
+            if ( µb.canFilterResponseData !== true ) { return; }
 
-        const stream = request.stream =
-            browser.webRequest.filterResponseData(extras.requestId);
-        stream.ondata = onStreamData;
-        stream.onstop = onStreamStop;
-        stream.onerror = onStreamError;
-        filterers.set(stream, request);
+            if ( (fctxt.itype & BINARY_TYPES) !== 0 ) { return; }
 
-        return true;
+            if ( fctxt.method !== fc.METHOD_GET ) {
+                if ( fctxt.method !== fc.METHOD_POST ) {
+                    return;
+                }
+            }
+
+            // https://github.com/gorhill/uBlock/issues/3478
+            const statusCode = details.statusCode || 0;
+            if ( statusCode === 0 ) { return; }
+
+            const hostname = fctxt.getHostname();
+            if ( hostname === '' ) { return; }
+
+            // https://bugzilla.mozilla.org/show_bug.cgi?id=1426789
+            const headers = details.responseHeaders;
+            const disposition = headerValueFromName('content-disposition', headers);
+            if ( disposition !== '' ) {
+                if ( disposition.startsWith('inline') === false ) { return; }
+            }
+
+            mime = 'text/plain';
+            charset = 'utf-8';
+            const contentType = headerValueFromName('content-type', headers) ||
+                contentTypeFromDetails(details);
+            if ( contentType !== '' ) {
+                mime = mimeFromContentType(contentType);
+                if ( mime === undefined ) { return; }
+                if ( mime.startsWith('text/') === false ) {
+                    if ( otherValidMimes.has(mime) === false ) { return; }
+                }
+                charset = charsetFromContentType(contentType);
+                if ( charset !== undefined ) {
+                    charset = textEncode.normalizeCharset(charset);
+                    if ( charset === undefined ) { return; }
+                } else {
+                    charset = charsetFromMime(mime);
+                }
+            }
+
+            return true;
+        }
     };
 })();
 
@@ -991,7 +1056,7 @@ const injectCSP = function(fctxt, pageStore, responseHeaders) {
 
     if ( cspSubsets.length === 0 ) { return; }
 
-    µb.updateToolbarIcon(fctxt.tabId, 0x02);
+    µb.updateToolbarIcon(fctxt.tabId, 0b0010);
 
     // Use comma to merge CSP directives.
     // Ref.: https://www.w3.org/TR/CSP2/#implementation-considerations
@@ -1012,18 +1077,31 @@ const injectCSP = function(fctxt, pageStore, responseHeaders) {
 
 /******************************************************************************/
 
-// https://github.com/uBlockOrigin/uBlock-issues/issues/1553
-// https://github.com/WICG/floc#opting-out-of-computation
-
-const foilFloc = function(fctxt, responseHeaders) {
-    const hn = fctxt.getHostname();
-    if ( scriptletFilteringEngine.hasScriptlet(hn, 1, 'no-floc') === false ) {
-        return false;
+const injectPP = function(fctxt, pageStore, responseHeaders) {
+    const permissions = [];
+    const directives = staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'permissions');
+    if ( directives !== undefined ) {
+        for ( const directive of directives ) {
+            if ( directive.result !== 1 ) { continue; }
+            permissions.push(directive.value.replace('|', ', '));
+        }
     }
+
+    if ( logger.enabled && directives !== undefined ) {
+        fctxt.setRealm('network')
+             .pushFilters(directives.map(a => a.logData()))
+             .toLogger();
+    }
+
+    if ( permissions.length === 0 ) { return; }
+
+    µb.updateToolbarIcon(fctxt.tabId, 0x02);
+
     responseHeaders.push({
-        name: 'Permissions-Policy',
-        value: 'interest-cohort=()' }
-    );
+        name: 'permissions-policy',
+        value: permissions.join(', ')
+    });
+
     return true;
 };
 
@@ -1080,7 +1158,9 @@ const headerValueFromName = function(headerName, headers) {
 
 const strictBlockBypasser = {
     hostnameToDeadlineMap: new Map(),
-    cleanupTimer: undefined,
+    cleanupTimer: vAPI.defer.create(( ) => {
+        strictBlockBypasser.cleanup();
+    }),
 
     cleanup: function() {
         for ( const [ hostname, deadline ] of this.hostnameToDeadlineMap ) {
@@ -1090,35 +1170,23 @@ const strictBlockBypasser = {
         }
     },
 
+    revokeTime: function() {
+        return Date.now() + µb.hiddenSettings.strictBlockingBypassDuration * 1000;
+    },
+
     bypass: function(hostname) {
         if ( typeof hostname !== 'string' || hostname === '' ) { return; }
-        this.hostnameToDeadlineMap.set(
-            hostname,
-            Date.now() + µb.hiddenSettings.strictBlockingBypassDuration * 1000
-        );
+        this.hostnameToDeadlineMap.set(hostname, this.revokeTime());
     },
 
     isBypassed: function(hostname) {
         if ( this.hostnameToDeadlineMap.size === 0 ) { return false; }
-        let bypassDuration =
-            µb.hiddenSettings.strictBlockingBypassDuration * 1000;
-        if ( this.cleanupTimer === undefined ) {
-            this.cleanupTimer = vAPI.setTimeout(
-                ( ) => {
-                    this.cleanupTimer = undefined;
-                    this.cleanup();
-                },
-                bypassDuration + 10000
-            );
-        }
+        this.cleanupTimer.on({ sec: µb.hiddenSettings.strictBlockingBypassDuration + 10 });
         for (;;) {
             const deadline = this.hostnameToDeadlineMap.get(hostname);
             if ( deadline !== undefined ) {
                 if ( deadline > Date.now() ) {
-                    this.hostnameToDeadlineMap.set(
-                        hostname,
-                        Date.now() + bypassDuration
-                    );
+                    this.hostnameToDeadlineMap.set(hostname, this.revokeTime());
                     return true;
                 }
                 this.hostnameToDeadlineMap.delete(hostname);
@@ -1147,7 +1215,7 @@ const webRequest = {
             vAPI.net.suspend();
         }
 
-        return async ( ) => {
+        return ( ) => {
             vAPI.net.setSuspendableListener(onBeforeRequest);
             vAPI.net.addListener(
                 'onHeadersReceived',
@@ -1158,6 +1226,7 @@ const webRequest = {
             vAPI.net.addListener(
                 'onResponseStarted',
                 details => {
+                    if ( details.tabId === -1 ) { return; }
                     const pageStore = µb.pageStoreFromTabId(details.tabId);
                     if ( pageStore === null ) { return; }
                     if ( pageStore.getNetFilteringSwitch() === false ) { return; }
@@ -1168,6 +1237,14 @@ const webRequest = {
                     urls: [ 'http://*/*', 'https://*/*' ]
                 }
             );
+            vAPI.defer.once({ sec: µb.hiddenSettings.toolbarWarningTimeout }).then(( ) => {
+                if ( vAPI.net.hasUnprocessedRequest() === false ) { return; }
+                vAPI.net.removeUnprocessedRequest();
+                return vAPI.tabs.getCurrent();
+            }).then(tab => {
+                if ( tab instanceof Object === false ) { return; }
+                µb.updateToolbarIcon(tab.id, 0b0110);
+            });
             vAPI.net.unsuspend({ all: true });
         };
     })(),
