@@ -2,6 +2,8 @@
 #
 # This script assumes a linux environment
 
+set -e
+
 echo "*** uBlock: Cleaning."
-rm -R dist/build
+rm -Rf dist/build
 echo "*** uBlock: Cleaned."
